@@ -39,6 +39,7 @@
 #include <gtk/gtkmain.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-mime-monitor.h>
 #include <libxml/parser.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -584,6 +585,51 @@ nautilus_directory_set_up_request (Request *request,
 
 }
 
+static void
+mime_db_changed_callback (GnomeVFSMIMEMonitor *ignore, NautilusDirectory *dir)
+{
+	const Monitor *monitor;
+	GList *ptr, *attrs;
+	GList *file_list;
+
+	g_return_if_fail (dir != NULL);
+	g_return_if_fail (dir->details != NULL);
+
+	attrs = NULL;
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_CAPABILITIES);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_METADATA);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_FILE_TYPE);
+	attrs = g_list_prepend (attrs, NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES);
+
+	file_list = NULL;
+	for (ptr = dir->details->monitor_list ; ptr != NULL ; ptr = ptr->next) {
+		monitor = ptr->data;
+		if (monitor->request.file_info && monitor->file != NULL) {
+			if (nautilus_file_is_self_owned (monitor->file)) {
+				nautilus_file_emit_changed (monitor->file);
+				nautilus_file_invalidate_attributes (monitor->file, attrs);
+			} else {
+				file_list = g_list_prepend (file_list, 
+							    monitor->file);
+			}
+			
+		}
+	}
+	
+	if (file_list) {
+		nautilus_directory_emit_change_signals (dir, file_list);
+
+		for (ptr = file_list; ptr != NULL; ptr = ptr->next) {
+			nautilus_file_invalidate_attributes (ptr->data, attrs);
+		}
+		g_list_free (file_list);
+	}
+	g_list_free (attrs);
+}
+
 void
 nautilus_directory_monitor_add_internal (NautilusDirectory *directory,
 					 NautilusFile *file,
@@ -641,6 +687,13 @@ nautilus_directory_monitor_add_internal (NautilusDirectory *directory,
 		nautilus_directory_register_metadata_monitor (directory);	
 	}
 
+	if (monitor->request.file_info && directory->details->mime_db_monitor == 0) {
+		directory->details->mime_db_monitor = g_signal_connect_object (
+			gnome_vfs_mime_monitor_get (),
+			"data_changed",
+			G_CALLBACK (mime_db_changed_callback), directory, 0);			  
+	}
+
 	/* Put the monitor file or all the files on the work queue. */
 	if (file != NULL) {
 		nautilus_directory_add_file_to_work_queue (directory, file);
@@ -1630,13 +1683,50 @@ wants_mime_list (const Request *request)
 	return request->mime_list;
 }
 
+static gboolean
+should_look_for_dot_directory_file (NautilusFile *file)
+{
+	char *uri;
+	guint i;
+
+	const char *schemes [] = {
+		"preferences:",
+		"all-preferences:",
+		"system-settings:",
+		"server-settings:",
+		"favorites:",
+		"start-here:",
+		"applications:",
+		"all-applications:"
+	};
+
+	uri = file->details->directory->details->uri;
+
+	/* We special-case the file shcme so we won't have
+	 * to go through the list every time
+	 */
+	if (eel_str_has_prefix (uri, "file:")) {
+		return FALSE;
+	}
+
+	for (i = 0; i < G_N_ELEMENTS (schemes); i++) {
+		if (eel_str_has_prefix (uri, schemes[i])) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static gboolean
 lacks_link_info (NautilusFile *file)
 {
 	if (file->details->file_info_is_up_to_date && 
 	    !file->details->link_info_is_up_to_date) {
-		if (nautilus_file_is_nautilus_link (file) ||
-		    nautilus_file_is_directory (file)) {
+		if (nautilus_file_is_nautilus_link (file)) {
+			return TRUE;
+		} else if (nautilus_file_is_directory (file) &&
+			should_look_for_dot_directory_file (file)) {
 			return TRUE;
 		} else {
 			link_info_done (file->details->directory, file, NULL, NULL, NULL);
@@ -2157,7 +2247,7 @@ deep_count_one (NautilusDirectory *directory,
 
 		/* Record the fact that we have to descend into this directory. */
 		escaped_name = gnome_vfs_escape_string (info->name);
-		uri = nautilus_make_path (directory->details->deep_count_uri, escaped_name);
+		uri = g_build_filename (directory->details->deep_count_uri, escaped_name, NULL);
 		g_free (escaped_name);
 		directory->details->deep_count_subdirectories = g_list_prepend
 			(directory->details->deep_count_subdirectories, uri);
@@ -2441,7 +2531,7 @@ mime_list_start (NautilusDirectory *directory,
 	if (!nautilus_file_is_directory (file)) {
 		g_list_free (file->details->mime_list);
 		file->details->mime_list_failed = FALSE;
-		file->details->got_directory_count = FALSE;
+		file->details->got_mime_list = FALSE;
 		file->details->mime_list_is_up_to_date = TRUE;
 
 		nautilus_directory_async_state_changed (directory);
@@ -2568,6 +2658,7 @@ top_left_start (NautilusDirectory *directory,
 
 	if (!nautilus_file_contains_text (file)) {
 		g_free (file->details->top_left_text);
+		file->details->top_left_text = NULL;
 		file->details->got_top_left_text = FALSE;
 		file->details->top_left_text_is_up_to_date = TRUE;
 
@@ -2734,14 +2825,21 @@ link_info_done (NautilusDirectory *directory,
 	file->details->got_link_info = TRUE;
 	g_free (file->details->activation_uri);
 	g_free (file->details->display_name);
-	g_free (file->details->custom_icon_uri);
+	g_free (file->details->custom_icon);
 	file->details->activation_uri = g_strdup (uri);
 	file->details->display_name = g_strdup (name);
-	file->details->custom_icon_uri = g_strdup (icon);
+	file->details->custom_icon = g_strdup (icon);
+ 	nautilus_file_clear_cached_display_name (file);
 
 	nautilus_directory_async_state_changed (directory);
 }
 
+static gboolean
+should_read_link_info_sync (NautilusFile *file)
+{
+	return (nautilus_file_is_local (file) &&
+		!nautilus_file_is_directory (file));
+}
 
 static void
 link_info_read_done (NautilusDirectory *directory,
@@ -2758,8 +2856,12 @@ link_info_read_done (NautilusDirectory *directory,
 	nautilus_file_ref (file);
 	link_info_done (directory, file, uri, name, icon);
 	nautilus_file_changed (file);
+
+	if (!should_read_link_info_sync (file)) {
+		async_job_end (directory, "link info");
+	}
+
 	nautilus_file_unref (file);
-	async_job_end (directory, "link info");
 }
 
 
@@ -2824,7 +2926,7 @@ make_dot_directory_uri (const char *uri)
 	 * directory's contents before all the scheduled calls of "look for .directory file" have been
 	 * finished. 
 	 */
-	if (gnome_vfs_uri_is_local (dot_dir_vfs_uri) && gnome_vfs_uri_exists (dot_dir_vfs_uri)) {
+	if (gnome_vfs_uri_is_local (dot_dir_vfs_uri)) {
 		dot_directory_uri = gnome_vfs_uri_to_string (dot_dir_vfs_uri, GNOME_VFS_URI_HIDE_NONE);
 	}
 	else {
@@ -2868,7 +2970,10 @@ link_info_start (NautilusDirectory *directory,
 {
 	char *uri, *dot_directory_uri = NULL;
 	gboolean nautilus_style_link, is_directory;
-
+	int file_size;
+	char *file_contents;
+	GnomeVFSResult result;
+	
 	if (directory->details->link_info_read_state != NULL) {
 		return;
 	}
@@ -2892,6 +2997,16 @@ link_info_start (NautilusDirectory *directory,
 	/* If it's not a link we are done. If it is, we need to read it. */
 	if (!(nautilus_style_link || (is_directory && dot_directory_uri != NULL) )) {
 		link_info_done (directory, file, NULL, NULL, NULL);
+	} else if (should_read_link_info_sync (file)) {
+		directory->details->link_info_read_state = g_new0 (LinkInfoReadState, 1);
+		directory->details->link_info_read_state->file = file;
+
+		result = eel_read_entire_file (uri, &file_size, &file_contents);
+		link_info_nautilus_link_read_callback (result, file_size, file_contents, directory);
+
+		/* We don't have to free file_contents here
+		 * because it's done in the callback function
+		 */
 	} else {
 		if (!async_job_start (directory, "link info")) {
 			g_free (dot_directory_uri);
@@ -3120,6 +3235,8 @@ nautilus_directory_cancel_loading_file_attributes (NautilusDirectory *directory,
 {
 	Request request;
 	
+	nautilus_directory_remove_file_from_work_queue (directory, file);
+
 	nautilus_directory_set_up_request (&request,
 					   file_attributes);
 
@@ -3201,6 +3318,8 @@ void
 nautilus_directory_add_file_to_work_queue (NautilusDirectory *directory,
 					   NautilusFile *file)
 {
+	g_return_if_fail (file->details->directory == directory);
+
 	if (!file_needs_work_done (directory, file)) {
 		return;
 	}
