@@ -33,7 +33,6 @@
 #include "nautilus-metadata.h"
 #include "nautilus-file-attributes.h"
 #include <eel/eel-string.h>
-#include "nautilus-theme.h"
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
@@ -81,12 +80,12 @@ desktop_background_realized (NautilusIconContainer *icon_container, void *discon
                                           
 	g_object_set_data (G_OBJECT (background), "icon_container", (gpointer) icon_container);
 
+	g_object_set_data (G_OBJECT (background), "screen",
+			   gtk_widget_get_screen (GTK_WIDGET (icon_container)));
+
 	nautilus_file_update_desktop_pixmaps (background);
 }
 
-static const char *default_theme_source = "directory";
-static const char *desktop_theme_source = "desktop";
-
 void
 nautilus_connect_desktop_background_to_file_metadata (NautilusIconContainer *icon_container,
                                                       NautilusFile *file)
@@ -95,7 +94,7 @@ nautilus_connect_desktop_background_to_file_metadata (NautilusIconContainer *ico
 
 	background = eel_get_widget_background (GTK_WIDGET (icon_container));
 
-	g_object_set_data (G_OBJECT (background), "theme_source", (gpointer) desktop_theme_source);
+	g_object_set_data (G_OBJECT (background), "is_desktop", (gpointer)1);
 
 	/* Strictly speaking, we don't need to know about metadata changes, since
 	 * the desktop setting aren't stored there. But, hooking up to metadata
@@ -120,18 +119,7 @@ nautilus_connect_desktop_background_to_file_metadata (NautilusIconContainer *ico
 static gboolean
 background_is_desktop (EelBackground *background)
 {
-	/* == works because we're carful to always use the same string.
-	 */
-	return g_object_get_data (G_OBJECT (background), "theme_source") == desktop_theme_source;
-}
-
-static const char *nautilus_file_background_peek_theme_source (EelBackground *background)
-{
-	char *theme_source;
-
-	theme_source = g_object_get_data (G_OBJECT (background), "theme_source");
-
-	return theme_source != NULL ? theme_source : default_theme_source;
+	return g_object_get_data (G_OBJECT (background), "is_desktop") != 0;
 }
 
 static GdkWindow *
@@ -143,104 +131,65 @@ background_get_desktop_background_window (EelBackground *background)
 	return layout != NULL ? GTK_LAYOUT (layout)->bin_window : NULL;
 }
 
-/* utility routine to handle mapping local image files in themes to a uri */
-static char*
-theme_image_path_to_uri (char *image_file, const char *theme_name)
-{
-	char *image_path;
-	char *image_uri;
-
-	if (image_file != NULL && !eel_istr_has_prefix (image_file, "file://")) {
-		
-		if (eel_str_has_prefix (image_file, "./")) {
-			image_path = nautilus_theme_get_image_path_from_theme (image_file + 2, theme_name);
-		} else {
-			image_path = g_strdup_printf ("%s/%s", NAUTILUS_DATADIR, image_file);
-		}
-		
-		if (image_path && g_file_test (image_path, G_FILE_TEST_EXISTS)) {
-			image_uri = gnome_vfs_get_uri_from_local_path (image_path);
-		} else {
-			image_uri = NULL;
-		}
-		
-		g_free (image_path);
-	} else {
-		image_uri = g_strdup (image_file);
-	}
-
-	return image_uri;
-}
- 
 static void
-nautilus_file_background_get_default_settings_for_theme (const char* theme_name,
-							  const char* theme_source,
-							  char **color,
-							  char **image,
-							  EelBackgroundImagePlacement *placement)
-{
-	char *image_local_path;
-
-	if (placement != NULL) {
-		*placement = EEL_BACKGROUND_TILED;
-	}
-
-	if (color != NULL) {
-		*color = nautilus_theme_get_theme_data_from_theme (theme_source, NAUTILUS_METADATA_KEY_LOCATION_BACKGROUND_COLOR, theme_name);
-	}
-
-	if (image != NULL) {
-		image_local_path = nautilus_theme_get_theme_data_from_theme (theme_source, NAUTILUS_METADATA_KEY_LOCATION_BACKGROUND_IMAGE, theme_name);
-		*image = theme_image_path_to_uri (image_local_path, theme_name);
-		g_free (image_local_path);
-	}
-}
-
-static void
-nautilus_file_background_get_default_settings (const char* theme_source,
-                                               char **color,
+nautilus_file_background_get_default_settings (char **color,
                                                char **image,
                                                EelBackgroundImagePlacement *placement)
 {
-	char *theme_name;
-	theme_name = nautilus_theme_get_theme ();
-	nautilus_file_background_get_default_settings_for_theme
-		(theme_name, theme_source, color, image, placement);
-	g_free (theme_name);
+        static gboolean setup_autos = FALSE;
+        static gboolean background_set;
+        static const char *background_color;
+        static const char *background_filename;
+
+        if (!setup_autos) {
+                setup_autos = TRUE;
+                eel_preferences_add_auto_boolean
+                        (NAUTILUS_PREFERENCES_BACKGROUND_SET,
+                         &background_set);
+                eel_preferences_add_auto_string
+                        (NAUTILUS_PREFERENCES_BACKGROUND_COLOR,
+                         &background_color);
+                eel_preferences_add_auto_string
+                        (NAUTILUS_PREFERENCES_BACKGROUND_FILENAME,
+                         &background_filename);
+        }
+
+        if (color) {
+                *color = background_set ? g_strdup (background_color) : NULL;
+        }
+
+        if (image) {
+                *image = background_set ? g_strdup (background_filename) : NULL;
+        }
+
+        if (placement) {
+                *placement = EEL_BACKGROUND_TILED;
+        }
 }
 
+
 static void
 nautilus_file_background_read_desktop_settings (char **color,
                                                 char **image,
                                                 EelBackgroundImagePlacement *placement)
 {
-	char*	 default_image_uri;
-	EelBackgroundImagePlacement default_placement;
-	
 	char	*end_color;
 	char	*start_color;
-	char	*default_color;
 	gboolean use_gradient;
 	gboolean is_horizontal;
 
-	char *theme_name;
 	BGPreferences *prefs;
 
         prefs = BG_PREFERENCES (bg_preferences_new ());
 
 	bg_preferences_load (prefs);
 
-	theme_name = nautilus_theme_get_theme ();
-
-	nautilus_file_background_get_default_settings_for_theme
-		(theme_name, desktop_theme_source, &default_color, &default_image_uri, &default_placement);
-
         if (prefs->wallpaper_enabled) {
                 if (prefs->wallpaper_filename != NULL &&
                     prefs->wallpaper_filename [0] != '\0') {
                         *image = gnome_vfs_get_uri_from_local_path (prefs->wallpaper_filename);
                 } else {
-                        *image = g_strdup (default_image_uri);
+                        *image = NULL;
                 }
 	}
         else {
@@ -283,10 +232,6 @@ nautilus_file_background_read_desktop_settings (char **color,
 	g_free (start_color);
 	g_free (end_color);
 
-	g_free (theme_name);	
-	g_free (default_color);
-	g_free (default_image_uri);
-
 	g_object_unref (prefs);
 }
 
@@ -295,7 +240,7 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, EelBa
 {
 	char *end_color;
 	char *start_color;
-	char *default_image_uri;
+        char *original_filename;
 
 	wallpaper_type_t wallpaper_align;
 	BGPreferences *prefs;
@@ -330,7 +275,7 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, EelBa
 		prefs->orientation = ORIENTATION_SOLID;
 	}
 
-	g_free (prefs->wallpaper_filename);
+        original_filename = prefs->wallpaper_filename;
 	if (image != NULL) {
 		prefs->wallpaper_filename = gnome_vfs_get_local_path_from_uri (image);
                 prefs->wallpaper_enabled = TRUE;
@@ -356,12 +301,9 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, EelBa
 		prefs->wallpaper_type = wallpaper_align;
 	} else {
                 prefs->wallpaper_enabled = FALSE;
-                /* Need to set something, or libbackground will barf */
-                nautilus_file_background_get_default_settings
-			(desktop_theme_source, NULL, &default_image_uri, NULL);
-		prefs->wallpaper_filename = gnome_vfs_get_local_path_from_uri (default_image_uri);
-                g_free (default_image_uri);
-	}
+                prefs->wallpaper_filename = g_strdup (original_filename);
+        }
+        g_free (original_filename);
 
 	bg_preferences_save (prefs);
 	g_object_unref (prefs);
@@ -370,11 +312,33 @@ nautilus_file_background_write_desktop_settings (char *color, char *image, EelBa
 static void
 nautilus_file_background_write_desktop_default_settings ()
 {
-	char *color;
-	char *image;
-	EelBackgroundImagePlacement placement;
-	nautilus_file_background_get_default_settings (desktop_theme_source, &color, &image, &placement);
-	nautilus_file_background_write_desktop_settings (color, image, placement);
+	/* We just unset all the gconf keys so they go back to
+	 * defaults
+	 */
+	GConfClient *client;
+	GConfChangeSet *set;
+
+	client = gconf_client_get_default ();
+	set = gconf_change_set_new ();
+
+	/* the list of keys here has to be kept in sync with libgnome
+	 * schemas, which isn't the most maintainable thing ever.
+	 */
+ 	gconf_change_set_unset (set, "/desktop/gnome/background/picture_options");
+	gconf_change_set_unset (set, "/desktop/gnome/background/picture_filename");
+	gconf_change_set_unset (set, "/desktop/gnome/background/picture_opacity");
+	gconf_change_set_unset (set, "/desktop/gnome/background/primary_color");
+	gconf_change_set_unset (set, "/desktop/gnome/background/secondary_color");
+	gconf_change_set_unset (set, "/desktop/gnome/background/color_shading_type");
+
+	/* this isn't atomic yet so it'll be a bit inefficient, but
+	 * someday it might be atomic.
+	 */
+ 	gconf_client_commit_change_set (client, set, FALSE, NULL);
+
+	gconf_change_set_unref (set);
+	
+	g_object_unref (G_OBJECT (client));
 }
 
 static int
@@ -422,15 +386,19 @@ nautilus_file_background_receive_gconf_changes (EelBackground *background)
  * (copied from gnome-source/control-panels/capplets/background-properties/render-background.c)
  */
 static GdkPixmap *
-make_root_pixmap (gint width, gint height)
+make_root_pixmap (GdkScreen *screen, gint width, gint height)
 {
 	Display *display;
-        char *display_name;
+        const char *display_name;
 	Pixmap result;
+	GdkPixmap *gdk_pixmap;
+	int screen_num;
+
+	screen_num = gdk_screen_get_number (screen);
 
 	gdk_flush ();
 
-        display_name = DisplayString (GDK_DISPLAY ());
+	display_name = gdk_display_get_name (gdk_screen_get_display (screen));
 	display = XOpenDisplay (display_name);
 
         if (display == NULL) {
@@ -442,13 +410,17 @@ make_root_pixmap (gint width, gint height)
 	XSetCloseDownMode (display, RetainPermanent);
 
 	result = XCreatePixmap (display,
-				DefaultRootWindow (display),
+				RootWindow (display, screen_num),
 				width, height,
-				DefaultDepthOfScreen (DefaultScreenOfDisplay (GDK_DISPLAY())));
+				DefaultDepth (display, screen_num));
 
 	XCloseDisplay (display);
 
-	return gdk_pixmap_foreign_new (result);
+	gdk_pixmap = gdk_pixmap_foreign_new (result);
+	gdk_drawable_set_colormap (GDK_DRAWABLE (gdk_pixmap),
+				   gdk_drawable_get_colormap (gdk_screen_get_root_window (screen)));
+
+	return gdk_pixmap;
 }
 
 /* Set the root pixmap, and properties pointing to it. We
@@ -459,21 +431,26 @@ make_root_pixmap (gint width, gint height)
  * (copied from gnome-source/control-panels/capplets/background-properties/render-background.c)
  */
 static void 
-set_root_pixmap (GdkPixmap *pixmap)
+set_root_pixmap (GdkPixmap *pixmap, GdkScreen *screen)
 {
-	int     result;
-	gint    format;
-	gulong  nitems;
-	gulong  bytes_after;
-	guchar *data_esetroot;
-	Pixmap  pixmap_id;
-	Atom type;
+	int      result;
+	gint     format;
+	gulong   nitems;
+	gulong   bytes_after;
+	guchar  *data_esetroot;
+	Pixmap   pixmap_id;
+	Atom     type;
+	Display *display;
+	int      screen_num;
+
+	screen_num = gdk_screen_get_number (screen);
 
 	data_esetroot = NULL;
+	display = GDK_DISPLAY_XDISPLAY (gdk_screen_get_display (screen));
 
-	XGrabServer (GDK_DISPLAY());
+	XGrabServer (display);
 
-	result = XGetWindowProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+	result = XGetWindowProperty (display, RootWindow (display, screen_num),
 				     gdk_x11_get_xatom_by_name ("ESETROOT_PMAP_ID"),
 				     0L, 1L, False, XA_PIXMAP,
 				     &type, &format, &nitems, &bytes_after,
@@ -482,7 +459,7 @@ set_root_pixmap (GdkPixmap *pixmap)
 	if (data_esetroot != NULL) {
 		if (result == Success && type == XA_PIXMAP && format == 32 && nitems == 1) {
 			gdk_error_trap_push ();
-			XKillClient(GDK_DISPLAY(), *(Pixmap *)data_esetroot);
+			XKillClient (display, *(Pixmap *)data_esetroot);
 			gdk_flush ();
 			gdk_error_trap_pop ();
 		}
@@ -491,21 +468,21 @@ set_root_pixmap (GdkPixmap *pixmap)
 
 	pixmap_id = GDK_WINDOW_XWINDOW (pixmap);
 
-	XChangeProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+	XChangeProperty (display, RootWindow (display, screen_num),
 			 gdk_x11_get_xatom_by_name ("ESETROOT_PMAP_ID"), XA_PIXMAP,
 			 32, PropModeReplace,
 			 (guchar *) &pixmap_id, 1);
-	XChangeProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+	XChangeProperty (display, RootWindow (display, screen_num),
 			 gdk_x11_get_xatom_by_name ("_XROOTPMAP_ID"), XA_PIXMAP,
 			 32, PropModeReplace,
 			 (guchar *) &pixmap_id, 1);
 
-	XSetWindowBackgroundPixmap (GDK_DISPLAY (), GDK_ROOT_WINDOW (), pixmap_id);
-	XClearWindow (GDK_DISPLAY (), GDK_ROOT_WINDOW ());
+	XSetWindowBackgroundPixmap (display, RootWindow (display, screen_num), pixmap_id);
+	XClearWindow (display, RootWindow (display, screen_num));
 
-	XUngrabServer (GDK_DISPLAY());
+	XUngrabServer (display);
 	
-	XFlush(GDK_DISPLAY());
+	XFlush (display);
 }
 	
 static void
@@ -516,6 +493,7 @@ image_loading_done_callback (EelBackground *background, gboolean successful_load
 	GdkGC        *gc;
 	GdkPixmap    *pixmap;
 	GdkWindow    *background_window;
+	GdkScreen    *screen;
 
         if (GPOINTER_TO_INT (disconnect_signal)) {
 		g_signal_handlers_disconnect_by_func
@@ -524,10 +502,13 @@ image_loading_done_callback (EelBackground *background, gboolean successful_load
                          disconnect_signal);
 	}
 
-	width  = gdk_screen_width ();
-	height = gdk_screen_height ();
+	screen = g_object_get_data (G_OBJECT (background), "screen");
+	if (screen == NULL)
+		return;
+	width = gdk_screen_get_width (screen);
+	height = gdk_screen_get_height (screen);
 
-	pixmap = make_root_pixmap (width, height);
+	pixmap = make_root_pixmap (screen, width, height);
         if (pixmap == NULL) {
                 return;
         }
@@ -536,12 +517,12 @@ image_loading_done_callback (EelBackground *background, gboolean successful_load
 	eel_background_draw_to_drawable (background, pixmap, gc, 0, 0, width, height, width, height);
 	g_object_unref (gc);
 
-	set_root_pixmap (pixmap);
+	set_root_pixmap (pixmap, screen);
 
 	background_window = background_get_desktop_background_window (background);
-	if (background_window != NULL) {
+	if (background_window != NULL &&
+            gdk_drawable_get_depth (background_window) == gdk_drawable_get_depth (pixmap))
 		gdk_window_set_back_pixmap (background_window, pixmap, FALSE);
-	}
 
         g_object_unref (pixmap);
 }
@@ -567,13 +548,9 @@ nautilus_file_background_matches_default_settings (
 	gboolean match_color;
 	gboolean match_image;
 
-	/* A NULL default color or image is not considered when determining a match.
-	 */
+	match_color = (eel_strcmp (color, default_color) == 0);
 	
-	match_color = (default_color == NULL) || eel_strcmp (color, default_color) == 0;
-	
-	match_image = (default_image == NULL) ||
-		      ((eel_strcmp (image, default_image) == 0) && (placement == default_placement));
+	match_image = (eel_strcmp (image, default_image) == 0) && (placement == default_placement);
 	return match_color && match_image;
 }
 
@@ -596,7 +573,6 @@ nautilus_file_background_is_set (EelBackground *background)
 	placement = eel_background_get_image_placement (background);
 	
 	nautilus_file_background_get_default_settings (
-		nautilus_file_background_peek_theme_source (background),
 		&default_color, &default_image, &default_placement);
 		 			    
 	matches = nautilus_file_background_matches_default_settings (color, default_color,
@@ -685,9 +661,8 @@ initialize_background_from_settings (NautilusFile *file,
 
 		/* if there's none, read the default from the theme */
 		if (color == NULL && image == NULL) {
-			nautilus_file_background_get_default_settings 
-                                (nautilus_file_background_peek_theme_source (background),
-                                 &color, &image, &placement);	
+			nautilus_file_background_get_default_settings
+                                (&color, &image, &placement);	
 		}
 	}
 
@@ -859,8 +834,13 @@ nautilus_connect_background_to_file_metadata (GtkWidget    *widget,
 
 		/* arrange for notification when the theme changes */
 		eel_preferences_add_callback (NAUTILUS_PREFERENCES_THEME,
-                                              nautilus_file_background_theme_changed, background);	
-
+                                              nautilus_file_background_theme_changed, background);
+		eel_preferences_add_callback (NAUTILUS_PREFERENCES_BACKGROUND_SET,
+                                              nautilus_file_background_theme_changed, background);
+		eel_preferences_add_callback (NAUTILUS_PREFERENCES_BACKGROUND_COLOR,
+                                              nautilus_file_background_theme_changed, background);
+		eel_preferences_add_callback (NAUTILUS_PREFERENCES_BACKGROUND_FILENAME,
+                                              nautilus_file_background_theme_changed, background);
 	}
 
         /* Update the background based on the file metadata. */
