@@ -42,7 +42,6 @@
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
-#include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-pango-extensions.h>
@@ -1268,7 +1267,8 @@ nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 					   const char                  *embedded_text,
 					   guint                        nominal_size,
 					   NautilusEmblemAttachPoints  *attach_points,
-					   gboolean                     wants_default)
+					   gboolean                     wants_default,
+					   char                       **display_name)
 {
 	NautilusIconFactory *factory;
 	CacheIcon *cached_icon;
@@ -1295,6 +1295,15 @@ nautilus_icon_factory_get_pixbuf_for_icon (const char                  *icon,
 			attach_points->num_points = 0;
 		}
 	}
+
+	if (display_name != NULL) {
+		if (cached_icon->icon_data != NULL &&
+		    cached_icon->icon_data->display_name != NULL) {
+			*display_name = g_strdup (cached_icon->icon_data->display_name);
+		} else {
+			*display_name = NULL;
+		}
+	}
 	
 	/* if we don't want a default icon and one is returned, return NULL instead */
 	if (!wants_default && cached_icon == factory->fallback_icon) {
@@ -1396,7 +1405,7 @@ nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
 	pixbuf = nautilus_icon_factory_get_pixbuf_for_icon (icon, modifier,
 							    embedded_text,
 							    size_in_pixels,
-							    NULL, TRUE);
+							    NULL, TRUE, NULL);
 	
 	g_free (icon);
 
@@ -1407,12 +1416,13 @@ nautilus_icon_factory_get_pixbuf_for_file (NautilusFile *file,
 GdkPixbuf *
 nautilus_icon_factory_get_pixbuf_from_name (const char *icon_name,
 					    const char *modifier,
-					    guint size_in_pixels)
+					    guint size_in_pixels,
+					    char **display_name)
 {
 	return nautilus_icon_factory_get_pixbuf_for_icon (icon_name, modifier,
 							  NULL,
 							  size_in_pixels,
-							  NULL, TRUE);
+							  NULL, TRUE, display_name);
 }
 									  
 GdkPixbuf *
