@@ -45,6 +45,7 @@ typedef struct {
 	double x, y;
 	
 	/* Scale factor (stretches icon). */
+	/* ALEX TODO: only allow once scale */
 	double scale_x, scale_y;
 
 	/* Whether this item is selected. */
@@ -62,7 +63,7 @@ typedef struct {
 
 	double start_x, start_y;
 
-	GnomeCanvasItem *selection_rectangle;
+	EelCanvasItem *selection_rectangle;
 
 	guint timer_id;
 
@@ -100,8 +101,10 @@ typedef struct {
 enum {
 	LABEL_COLOR,
 	LABEL_COLOR_HIGHLIGHT,
+	LABEL_COLOR_ACTIVE,
 	LABEL_INFO_COLOR,
 	LABEL_INFO_COLOR_HIGHLIGHT,
+	LABEL_INFO_COLOR_ACTIVE,
 	LAST_LABEL_COLOR
 };
 
@@ -156,7 +159,7 @@ struct NautilusIconContainerDetails {
 
 	/* Renaming Details */
 	gboolean renaming;
-	GnomeIconTextItem *rename_widget;	/* Editable text item */
+	GtkWidget *rename_widget;	/* Editable text item */
 	char *original_text;			/* Copy of editable text for later compare */
 
 	/* typeahead selection state */
@@ -182,8 +185,8 @@ struct NautilusIconContainerDetails {
 
 	/* pixbuf and color for label highlighting */
 	GdkPixbuf *highlight_frame;
-	GdkColor   highlight_color;
 	guint32    highlight_color_rgba;
+	guint32    active_color_rgba;
 	
 	/* colors for text labels */
 	GdkGC   *label_gcs    [LAST_LABEL_COLOR];
@@ -224,6 +227,7 @@ struct NautilusIconContainerDetails {
 
 	/* Whether we should use drop shadows for the icon labels or not */
 	gboolean use_drop_shadows;
+	gboolean drop_shadows_requested;
 };
 
 /* Private functions shared by mutiple files. */
