@@ -32,6 +32,7 @@
 #include "nautilus-mime-actions.h"
 #include "nautilus-program-choosing.h"
 #include "nautilus-view-identifier.h"
+#include "egg-screen-help.h"
 #include <eel/eel-gnome-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -180,15 +181,13 @@ help_cb (GtkWidget *button, NautilusProgramChooser *program_chooser)
 		break;
 	}
 
-	gnome_help_display_desktop (NULL,
-				    "user-guide",
-				    "wgosnautilus.xml",
-				    section,
-				    &error);
+	egg_screen_help_display_desktop (
+			gtk_window_get_screen (GTK_WINDOW (program_chooser)),
+			NULL, "user-guide", "wgosnautilus.xml", section, &error);
 
 	if (error) {
 		GtkWidget *err_dialog;
-		err_dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (button)),
+		err_dialog = gtk_message_dialog_new (GTK_WINDOW (program_chooser),
 						     GTK_DIALOG_MODAL,
 						     GTK_MESSAGE_ERROR,
 						     GTK_BUTTONS_CLOSE,
@@ -961,13 +960,27 @@ set_default_for_item (ProgramFilePair *pair)
 }
 
 static void
-launch_mime_capplet (const char *mime_type)
+launch_mime_capplet (NautilusFile *file,
+		     GtkDialog    *parent_dialog)
 {
-	char *command;
+	GdkScreen *screen;
+	char *command, *tmp, *mime_type, *file_name;
+
+	screen = gtk_window_get_screen (GTK_WINDOW (parent_dialog));
+
+	tmp = nautilus_file_get_mime_type (file);
+	mime_type = g_shell_quote (tmp);
+	g_free (tmp);
+	tmp = nautilus_file_get_name (file);
+	file_name = g_shell_quote (tmp);
+	g_free (tmp);
+
+	command = g_strconcat (FILE_TYPES_CAPPLET_NAME, " ", mime_type, " ", file_name, NULL);
+	nautilus_launch_application_from_command (screen, FILE_TYPES_CAPPLET_NAME, command, NULL, FALSE);
 
-	command = g_strconcat (FILE_TYPES_CAPPLET_NAME, " ", mime_type, NULL);
-	nautilus_launch_application_from_command (FILE_TYPES_CAPPLET_NAME, command, NULL, FALSE);
 	g_free (command);
+	g_free (file_name);
+	g_free (mime_type);
 }
 
 static void
@@ -976,7 +989,7 @@ launch_mime_capplet_on_ok (GtkDialog *dialog, int response, gpointer callback_da
 	g_assert (GTK_IS_DIALOG (dialog));
 
 	if (response == GTK_RESPONSE_YES) {
-		launch_mime_capplet (callback_data);
+		launch_mime_capplet (callback_data, dialog);
 	}
 	gtk_object_destroy (GTK_OBJECT (dialog));
 }
@@ -985,16 +998,11 @@ static void
 launch_mime_capplet_and_close_dialog (GtkButton *button, gpointer callback_data)
 {
 	ProgramFilePair *file_pair;
-	char *mime_type;
-	
+
 	g_assert (GTK_IS_BUTTON (button));
 
 	file_pair = get_selected_program_file_pair (NAUTILUS_PROGRAM_CHOOSER (callback_data));
-	mime_type = nautilus_file_get_mime_type (file_pair->file);
-	launch_mime_capplet (mime_type);
-	
-	g_free (mime_type);
-
+ 	launch_mime_capplet (file_pair->file, GTK_DIALOG (callback_data));
 	gtk_dialog_response (GTK_DIALOG (callback_data),
 		GTK_RESPONSE_DELETE_EVENT);
 }
@@ -1608,9 +1616,9 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	dialog = eel_show_yes_no_dialog 
 		(prompt, dialog_title, _("Associate Application"), GTK_STOCK_CANCEL, parent_window);
 
-	g_signal_connect (dialog, "response",
-			  G_CALLBACK (launch_mime_capplet_on_ok),
-			  nautilus_file_get_mime_type (file));
+	g_signal_connect_object (dialog, "response",
+		G_CALLBACK (launch_mime_capplet_on_ok),
+		file, 0);
 
 	g_free (unavailable_message);
 	g_free (file_name);
