@@ -45,6 +45,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libnautilus-private/egg-screen-exec.h>
 #include <libnautilus-private/nautilus-bonobo-extensions.h>
 #include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-directory-notify.h>
@@ -85,6 +86,7 @@ struct FMDesktopIconViewDetails
 {
 	BonoboUIComponent *ui;
 	GList *mount_black_list;
+	GdkWindow *root_window;
 
 	/* For the desktop rescanning
 	 */
@@ -153,6 +155,7 @@ get_icon_container (FMDesktopIconView *icon_view)
 
 static void
 icon_container_set_workarea (NautilusIconContainer *icon_container,
+			     GdkScreen             *screen,
 			     long                  *workareas,
 			     int                    n_items)
 {
@@ -162,8 +165,8 @@ icon_container_set_workarea (NautilusIconContainer *icon_container,
 
 	left = right = top = bottom = 0;
 
-	screen_width  = gdk_screen_width ();
-	screen_height = gdk_screen_height ();
+	screen_width  = gdk_screen_get_width (screen);
+	screen_height = gdk_screen_get_height (screen);
 
 	for (i = 0; i < n_items; i += 4) {
 		int x      = workareas [i];
@@ -171,6 +174,9 @@ icon_container_set_workarea (NautilusIconContainer *icon_container,
 		int width  = workareas [i + 2];
 		int height = workareas [i + 3];
 
+		if ((x + width) > screen_width || (y + height) > screen_height)
+			continue;
+
 		left   = MAX (left, x);
 		right  = MAX (right, screen_width - width - x);
 		top    = MAX (top, y);
@@ -182,49 +188,48 @@ icon_container_set_workarea (NautilusIconContainer *icon_container,
 }
 
 static void
-net_workarea_changed (FMDesktopIconView *icon_view)
+net_workarea_changed (FMDesktopIconView *icon_view,
+		      GdkWindow         *window)
 {
 	long *workareas = NULL;
-	Atom type_returned;
+	GdkAtom type_returned;
 	int format_returned;
-	unsigned long items_returned;
-	unsigned long bytes_after_return;
+	int length_returned;
 	NautilusIconContainer *icon_container;
+	GdkScreen *screen;
 
 	g_return_if_fail (FM_IS_DESKTOP_ICON_VIEW (icon_view));
 
 	icon_container = get_icon_container (icon_view);
 
 	gdk_error_trap_push ();
-	if (XGetWindowProperty (GDK_DISPLAY (),
-				GDK_ROOT_WINDOW (),
-				gdk_x11_get_xatom_by_name ("_NET_WORKAREA"),
-				0, G_MAXLONG, False,
-				XA_CARDINAL,
-				&type_returned,
-				&format_returned,
-				&items_returned,
-				&bytes_after_return,
-				(unsigned char **)&workareas) != Success) {
-		if (workareas != NULL)
-			XFree (workareas);
+	if (!gdk_property_get (window,
+			       gdk_atom_intern ("_NET_WORKAREA", FALSE),
+			       gdk_x11_xatom_to_atom (XA_CARDINAL),
+			       0, G_MAXLONG, FALSE,
+			       &type_returned,
+			       &format_returned,
+			       &length_returned,
+			       (guchar **) &workareas)) {
 		workareas = NULL;
 	}
-			    
+
 	if (gdk_error_trap_pop ()
 	    || workareas == NULL
-	    || type_returned != XA_CARDINAL
-	    || (items_returned % 4) != 0
+	    || type_returned != gdk_x11_xatom_to_atom (XA_CARDINAL)
+	    || (length_returned % 4) != 0
 	    || format_returned != 32) {
 		nautilus_icon_container_set_margins (icon_container,
 						     0, 0, 0, 0);
 	} else {
+		screen = gdk_drawable_get_screen (GDK_DRAWABLE (window));
+
 		icon_container_set_workarea (
-				icon_container, workareas, items_returned);
+			icon_container, screen, workareas, length_returned / sizeof (long));
 	}
 
 	if (workareas != NULL)
-		XFree (workareas);
+		g_free (workareas);
 }
 
 static GdkFilterReturn
@@ -240,7 +245,7 @@ desktop_icon_view_property_filter (GdkXEvent *gdk_xevent,
 	switch (xevent->type) {
 	case PropertyNotify:
 		if (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("_NET_WORKAREA"))
-			net_workarea_changed (icon_view);
+			net_workarea_changed (icon_view, event->any.window);
 		break;
 	default:
 		break;
@@ -256,11 +261,6 @@ fm_desktop_icon_view_finalize (GObject *object)
 
 	icon_view = FM_DESKTOP_ICON_VIEW (object);
 
-	/* Remove the property filter */
-	gdk_window_remove_filter (gdk_get_default_root_window (),
-				  desktop_icon_view_property_filter,
-				  icon_view);
-
 	/* Remove desktop rescan timeout. */
 	if (icon_view->details->reload_desktop_timeout != 0) {
 		gtk_timeout_remove (icon_view->details->reload_desktop_timeout);
@@ -496,6 +496,38 @@ event_callback (GtkWidget *widget, GdkEvent *event, FMDesktopIconView *desktop_i
 {
 }
 
+static void
+unrealized_callback (GtkWidget *widget, FMDesktopIconView *desktop_icon_view)
+{
+	g_return_if_fail (desktop_icon_view->details->root_window != NULL);
+
+	/* Remove the property filter */
+	gdk_window_remove_filter (desktop_icon_view->details->root_window,
+				  desktop_icon_view_property_filter,
+				  desktop_icon_view);
+	desktop_icon_view->details->root_window = NULL;
+}
+
+static void
+realized_callback (GtkWidget *widget, FMDesktopIconView *desktop_icon_view)
+{
+	GdkWindow *root_window;
+
+	g_return_if_fail (desktop_icon_view->details->root_window == NULL);
+
+	root_window = gdk_screen_get_root_window (gtk_widget_get_screen (widget));
+
+	desktop_icon_view->details->root_window = root_window;
+
+	/* Read out the workarea geometry and update the icon container accordingly */
+	net_workarea_changed (desktop_icon_view, root_window);
+
+	/* Setup the property filter */
+	gdk_window_set_events (root_window, GDK_PROPERTY_NOTIFY);
+	gdk_window_add_filter (root_window,
+			       desktop_icon_view_property_filter,
+			       desktop_icon_view);
+}
 
 static NautilusZoomLevel
 get_default_zoom_level (void)
@@ -694,6 +726,10 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 				 G_CALLBACK (volume_mounted_callback), desktop_icon_view, 0);
 	g_signal_connect_object (nautilus_volume_monitor_get (), "volume_unmounted",
 				 G_CALLBACK (volume_unmounted_callback), desktop_icon_view, 0);
+	g_signal_connect_object (desktop_icon_view, "realize",
+				 G_CALLBACK (realized_callback), desktop_icon_view, 0);
+	g_signal_connect_object (desktop_icon_view, "unrealize",
+				 G_CALLBACK (unrealized_callback), desktop_icon_view, 0);
 	
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_HOME_URI,
 				      home_uri_changed,
@@ -709,32 +745,27 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	
 	default_zoom_level_changed (desktop_icon_view);
 	fm_desktop_icon_view_update_icon_container_fonts (desktop_icon_view);
-
-	/* Read out the workarea geometry and update the icon container accordingly */
-	net_workarea_changed (desktop_icon_view);
-
-	/* Setup the property filter */
-	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);
-
-	gdk_window_add_filter (gdk_get_default_root_window (),
-			       desktop_icon_view_property_filter,
-			       desktop_icon_view);
 }
 
 static void
 new_terminal_callback (BonoboUIComponent *component, gpointer data, const char *verb)
 {
-	eel_gnome_open_terminal (NULL);
+        g_assert (FM_DIRECTORY_VIEW (data));
+
+	eel_gnome_open_terminal_on_screen (NULL, gtk_widget_get_screen (GTK_WIDGET (data)));
 }
 
 static void
 new_launcher_callback (BonoboUIComponent *component, gpointer data, const char *verb)
 {
 	char *desktop_directory;
-	
+
+        g_assert (FM_DIRECTORY_VIEW (data));
+
 	desktop_directory = nautilus_get_desktop_directory ();
 
-	nautilus_launch_application_from_command ("gnome-desktop-item-edit", 
+	nautilus_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (data)),
+						  "gnome-desktop-item-edit", 
 						  "gnome-desktop-item-edit --create-new",
 						  desktop_directory, 
 						  FALSE);
@@ -747,9 +778,13 @@ change_background_callback (BonoboUIComponent *component,
 	  		    gpointer data, 
 			    const char *verb)
 {
-	nautilus_launch_application_from_command 
-		(_("Background"),
-		 "gnome-background-properties", NULL, FALSE);
+        g_assert (FM_DIRECTORY_VIEW (data));
+
+	nautilus_launch_application_from_command (gtk_widget_get_screen (GTK_WIDGET (data)),
+						  _("Background"),
+						  "gnome-background-properties",
+						  NULL,
+						  FALSE);
 }
 
 static void
@@ -767,33 +802,9 @@ reset_background_callback (BonoboUIComponent *component,
 			   gpointer data, 
 			   const char *verb)
 {
-	/* We just unset all the gconf keys so they go back to
-	 * defaults
-	 */
-	GConfClient *client;
-	GConfChangeSet *set;
-
-	client = gconf_client_get_default ();
-	set = gconf_change_set_new ();
-
-	/* the list of keys here has to be kept in sync with libgnome
-	 * schemas, which isn't the most maintainable thing ever.
-	 */
- 	gconf_change_set_unset (set, "/desktop/gnome/background/picture_options");
-	gconf_change_set_unset (set, "/desktop/gnome/background/picture_filename");
-	gconf_change_set_unset (set, "/desktop/gnome/background/picture_opacity");
-	gconf_change_set_unset (set, "/desktop/gnome/background/primary_color");
-	gconf_change_set_unset (set, "/desktop/gnome/background/secondary_color");
-	gconf_change_set_unset (set, "/desktop/gnome/background/color_shading_type");
-
-	/* this isn't atomic yet so it'll be a bit inefficient, but
-	 * someday it might be atomic.
-	 */
- 	gconf_client_commit_change_set (client, set, FALSE, NULL);
-
-	gconf_change_set_unref (set);
-	
-	g_object_unref (G_OBJECT (client));
+	eel_background_reset 
+		(fm_directory_view_get_background 
+		 (FM_DIRECTORY_VIEW (data)));
 }
 
 static gboolean
@@ -856,7 +867,7 @@ volume_ops_callback (BonoboUIComponent *component, gpointer data, const char *ve
 	gboolean status;
 	GError *error;
 	GtkWidget *dialog;
-
+	GdkScreen *screen;
 
         g_assert (FM_IS_DIRECTORY_VIEW (data));
         
@@ -940,13 +951,17 @@ volume_ops_callback (BonoboUIComponent *component, gpointer data, const char *ve
 
 		if (command) {
 			error = NULL;
-			status = g_spawn_command_line_async (command, &error);
+
+			screen = gtk_widget_get_screen (GTK_WIDGET (view));
+
+			status = egg_screen_execute_command_line_async (screen, command, &error);
 			if (!status) {
 				dialog = gtk_message_dialog_new (NULL, 0,
 								 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
 								 _("Error executing utility program '%s': %s"), command, error->message);
 				g_signal_connect (G_OBJECT (dialog), "response", 
 						  G_CALLBACK (gtk_widget_destroy), NULL);
+				gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 				gtk_widget_show (dialog);
 				g_error_free (error);
 			}
@@ -1077,7 +1092,7 @@ volume_mounted_callback (NautilusVolumeMonitor *monitor,
 static void
 unlink_and_notify (const char *path)
 {
-	char *uri;
+	char *uri, *unescaped_uri;
 	GList one_item_list;
 
 	unlink (path);
@@ -1086,8 +1101,11 @@ unlink_and_notify (const char *path)
 	if (uri == NULL) {
 		return;
 	}
+ 
+	unescaped_uri = gnome_vfs_unescape_string (uri, NULL);
+	g_free (uri);
 
-	one_item_list.data = uri;
+	one_item_list.data = unescaped_uri;
 	one_item_list.next = NULL;
 	one_item_list.prev = NULL;
 	nautilus_directory_notify_files_removed (&one_item_list);
