@@ -22,43 +22,30 @@
  * Authors: Darin Adler <darin@bentspoon.com>
  */
 
-/* nautilus-desktop-window.c
- */
-
 #include <config.h>
 #include "nautilus-desktop-window.h"
 
 #include <X11/Xatom.h>
-#include <eel/eel-gtk-extensions.h>
-#include <eel/eel-gtk-macros.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtklayout.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomevfs/gnome-vfs-find-directory.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnome/gnome-macros.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
-#include <libnautilus-private/nautilus-link.h>
 
 struct NautilusDesktopWindowDetails {
-	GList *unref_list;
+	int dummy;
 };
 
-static void nautilus_desktop_window_class_init (NautilusDesktopWindowClass *klass);
-static void nautilus_desktop_window_init       (NautilusDesktopWindow      *window);
-static void set_wmspec_desktop_hint            (GdkWindow                  *window);
+static void set_wmspec_desktop_hint (GdkWindow *window);
 
-EEL_CLASS_BOILERPLATE (NautilusDesktopWindow, nautilus_desktop_window, NAUTILUS_TYPE_WINDOW)
+GNOME_CLASS_BOILERPLATE (NautilusDesktopWindow, nautilus_desktop_window,
+			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
 
 static void
-nautilus_desktop_window_init (NautilusDesktopWindow *window)
+nautilus_desktop_window_instance_init (NautilusDesktopWindow *window)
 {
 	window->details = g_new0 (NautilusDesktopWindowDetails, 1);
 
-	gtk_widget_set_size_request (GTK_WIDGET (window),
-				     gdk_screen_width (),
-				     gdk_screen_height ());
-
 	gtk_window_move (GTK_WINDOW (window), 0, 0);
 
 	/* shouldn't really be needed given our semantic type
@@ -68,18 +55,6 @@ nautilus_desktop_window_init (NautilusDesktopWindow *window)
 				  FALSE);
 }
 
-static void
-nautilus_desktop_window_realized (NautilusDesktopWindow *window)
-{
-	/* Tuck the desktop windows xid in the root to indicate we own the desktop.
-	 */
-	Window window_xid;
-	window_xid = GDK_WINDOW_XWINDOW (GTK_WIDGET (window)->window);
-	gdk_property_change (NULL, gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
-			     gdk_x11_xatom_to_atom (XA_WINDOW), 32,
-			     PropModeReplace, (guchar *) &window_xid, 1);
-}
-
 static gint
 nautilus_desktop_window_delete_event (NautilusDesktopWindow *window)
 {
@@ -105,20 +80,27 @@ nautilus_desktop_window_update_directory (NautilusDesktopWindow *window)
 }
 
 NautilusDesktopWindow *
-nautilus_desktop_window_new (NautilusApplication *application)
+nautilus_desktop_window_new (NautilusApplication *application,
+			     GdkScreen           *screen)
 {
 	NautilusDesktopWindow *window;
+	int width_request, height_request;
+
+	width_request = gdk_screen_get_width (screen);
+	height_request = gdk_screen_get_height (screen);
 
 	window = NAUTILUS_DESKTOP_WINDOW
 		(gtk_widget_new (nautilus_desktop_window_get_type(),
 				 "app", application,
 				 "app_id", "nautilus",
+				 "width_request", width_request,
+				 "height_request", height_request,
+				 "screen", screen,
 				 NULL));
 
 	/* Special sawmill setting*/
 	gtk_window_set_wmclass (GTK_WINDOW (window), "desktop_window", "Nautilus");
 
-	g_signal_connect (window, "realize", G_CALLBACK (nautilus_desktop_window_realized), NULL);
 	g_signal_connect (window, "delete_event", G_CALLBACK (nautilus_desktop_window_delete_event), NULL);
 
 	/* Point window at the desktop folder.
@@ -126,8 +108,6 @@ nautilus_desktop_window_new (NautilusApplication *application)
 	 */
 	nautilus_desktop_window_update_directory (window);
 
-	gtk_widget_show (GTK_WIDGET (window));
-
 	return window;
 }
 
@@ -138,14 +118,21 @@ finalize (GObject *object)
 
 	window = NAUTILUS_DESKTOP_WINDOW (object);
 
-	gdk_property_delete (NULL, gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", TRUE));
-
-	eel_gtk_object_list_free (window->details->unref_list);
 	g_free (window->details);
 
-	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/* Disable this code for the time being, since its:
+ * a) not working
+ * b) crashes nautilus if the root window has a different
+ *    depth than the nautilus window
+ *
+ * The idea behind the code is to grab the old background pixmap
+ * and temporarily set it as the background for the nautilus window
+ * to avoid flashing before the correct background has been set.
+ */
+#if 0
 static void
 set_gdk_window_background (GdkWindow *window,
 			   gboolean   have_pixel,
@@ -244,10 +231,74 @@ set_window_background (GtkWidget *widget,
 	/* For both parent and child, if it's a layout then set on the
 	 * bin window as well.
 	 */
-	if (GTK_IS_LAYOUT (widget))
+	if (GTK_IS_LAYOUT (widget)) {
 		set_gdk_window_background (GTK_LAYOUT (widget)->bin_window,
 					   have_pixel,
 					   pixmap, pixel);
+	}
+}
+#endif
+static void
+map (GtkWidget *widget)
+{
+	/* Disable for now, see above for comments */
+#if 0
+	set_window_background (widget, FALSE, FALSE, None, 0);
+#endif	
+	/* Chain up to realize our children */
+	GTK_WIDGET_CLASS (parent_class)->map (widget);
+	gdk_window_lower (widget->window);
+}
+
+
+static void
+unrealize (GtkWidget *widget)
+{
+	NautilusDesktopWindow *window;
+	GdkWindow *root_window;
+
+	window = NAUTILUS_DESKTOP_WINDOW (widget);
+
+	root_window = gdk_screen_get_root_window (
+				gtk_window_get_screen (GTK_WINDOW (window)));
+
+	gdk_property_delete (root_window,
+			     gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", TRUE));
+
+	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
+static void
+set_wmspec_desktop_hint (GdkWindow *window)
+{
+	GdkAtom atom;
+
+	atom = gdk_atom_intern ("_NET_WM_WINDOW_TYPE_DESKTOP", FALSE);
+        
+	gdk_property_change (window,
+			     gdk_atom_intern ("_NET_WM_WINDOW_TYPE", FALSE),
+			     gdk_x11_xatom_to_atom (XA_ATOM), 32,
+			     GDK_PROP_MODE_REPLACE, (guchar *) &atom, 1);
+}
+
+static void
+set_desktop_window_id (NautilusDesktopWindow *window,
+		       GdkWindow             *gdkwindow)
+{
+	/* Tuck the desktop windows xid in the root to indicate we own the desktop.
+	 */
+	Window window_xid;
+	GdkWindow *root_window;
+
+	root_window = gdk_screen_get_root_window (
+				gtk_window_get_screen (GTK_WINDOW (window)));
+
+	window_xid = GDK_WINDOW_XWINDOW (gdkwindow);
+
+	gdk_property_change (root_window,
+			     gdk_atom_intern ("NAUTILUS_DESKTOP_WINDOW_ID", FALSE),
+			     gdk_x11_xatom_to_atom (XA_WINDOW), 32,
+			     GDK_PROP_MODE_REPLACE, (guchar *) &window_xid, 1);
 }
 
 static void
@@ -262,19 +313,12 @@ realize (GtkWidget *widget)
 			      | GDK_KEY_PRESS_MASK | GDK_KEY_PRESS_MASK);
 			      
 	/* Do the work of realizing. */
-	EEL_CALL_PARENT (GTK_WIDGET_CLASS, realize, (widget));
+	GTK_WIDGET_CLASS (parent_class)->realize (widget);
 
 	/* This is the new way to set up the desktop window */
 	set_wmspec_desktop_hint (widget->window);
-}
 
-static void
-map (GtkWidget *widget)
-{
-	set_window_background (widget, FALSE, FALSE, None, 0);
-	
-	/* Chain up to realize our children */
-	EEL_CALL_PARENT (GTK_WIDGET_CLASS, map, (widget));
+	set_desktop_window_id (window, widget->window);
 }
 
 static void
@@ -286,29 +330,15 @@ real_add_current_location_to_history_list (NautilusWindow *window)
 }
 
 static void
-set_wmspec_desktop_hint (GdkWindow *window)
+nautilus_desktop_window_class_init (NautilusDesktopWindowClass *class)
 {
-        Atom atom;
-        
-        atom = XInternAtom (gdk_display,
-                            "_NET_WM_WINDOW_TYPE_DESKTOP",
-                            False);
+	G_OBJECT_CLASS (class)->finalize = finalize;
+	GTK_WIDGET_CLASS (class)->realize = realize;
+	GTK_WIDGET_CLASS (class)->unrealize = unrealize;
 
-        XChangeProperty (GDK_WINDOW_XDISPLAY (window),
-                         GDK_WINDOW_XWINDOW (window),
-                         XInternAtom (gdk_display,
-                                      "_NET_WM_WINDOW_TYPE",
-                                      False),
-                         XA_ATOM, 32, PropModeReplace,
-                         (guchar *)&atom, 1);
-}
 
-static void
-nautilus_desktop_window_class_init (NautilusDesktopWindowClass *klass)
-{
-	G_OBJECT_CLASS (klass)->finalize = finalize;
-	GTK_WIDGET_CLASS (klass)->realize = realize;
-	GTK_WIDGET_CLASS (klass)->map = map;
-	NAUTILUS_WINDOW_CLASS (klass)->add_current_location_to_history_list 
+	GTK_WIDGET_CLASS (class)->map = map;
+
+	NAUTILUS_WINDOW_CLASS (class)->add_current_location_to_history_list 
 		= real_add_current_location_to_history_list;
 }
