@@ -62,8 +62,8 @@ struct NautilusWindowDetails
 
 	/* Toolbar. */
         GtkTooltips *tooltips;
-	BonoboUIToolbarButtonItem *back_button_item;
-	BonoboUIToolbarButtonItem *forward_button_item;
+	BonoboUIToolbarItem *back_button_item;
+	BonoboUIToolbarItem *forward_button_item;
 
         /* Current location. */
         char *location;
@@ -95,6 +95,10 @@ struct NautilusWindowDetails
 
         /* Location bar */
         gboolean temporary_navigation_bar;
+
+        /* Side Pane */
+        int side_pane_width;
+        GtkWidget *current_side_panel;
 };
 
 #define NAUTILUS_MENU_PATH_BACK_ITEM			"/menu/Go/Back"
@@ -144,7 +148,6 @@ void               nautilus_window_go_back                               (Nautil
 void               nautilus_window_go_forward                            (NautilusWindow    *window);
 void               nautilus_window_go_up                                 (NautilusWindow    *window);
 void               nautilus_window_update_find_menu_item                 (NautilusWindow    *window);
-void               nautilus_window_toolbar_remove_theme_callback         (NautilusWindow    *window);
 void               nautilus_window_remove_bookmarks_menu_callback        (NautilusWindow    *window);
 void               nautilus_window_remove_go_menu_callback               (NautilusWindow    *window);
 void               nautilus_window_remove_bookmarks_menu_items           (NautilusWindow    *window);
