@@ -32,13 +32,14 @@
 
 #include "nautilus-application.h"
 #include "nautilus-bookmarks-window.h"
+#include "nautilus-information-panel.h"
 #include "nautilus-main.h"
-#include "nautilus-sidebar.h"
 #include "nautilus-signaller.h"
 #include "nautilus-switchable-navigation-bar.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-zoom-control.h"
 #include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-property-bag-client.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
@@ -89,6 +90,10 @@
 #define STATUS_BAR_CLEAR_TIMEOUT 10000	/* milliseconds */
 #define MAX_HISTORY_ITEMS 50
 
+/* FIXME bugzilla.gnome.org 41245: hardwired sizes */
+#define SIDE_PANE_MINIMUM_WIDTH 1
+#define SIDE_PANE_MINIMUM_HEIGHT 400
+
 /* dock items */
 #define LOCATION_BAR_PATH	"/Location Bar"
 #define TOOLBAR_PATH            "/Toolbar"
@@ -111,6 +116,7 @@ enum {
 };
 
 static GList *history_list;
+static int side_pane_width_auto_value = SIDE_PANE_MINIMUM_WIDTH;
 
 static void update_sidebar_panels_from_preferences (NautilusWindow *window);
 static void sidebar_panels_changed_callback        (gpointer        user_data);
@@ -458,25 +464,28 @@ nautilus_window_zoom_to_fit (NautilusWindow *window)
  * (The user can still stretch the window taller if desired).
  */
 static guint
-get_max_forced_height (void)
+get_max_forced_height (GdkScreen *screen)
 {
-	return (gdk_screen_height () * 90) / 100;
+	return (gdk_screen_get_height (screen) * 90) / 100;
 }
 
 /* Code should never force the window wider than this size.
  * (The user can still stretch the window wider if desired).
  */
 static guint
-get_max_forced_width (void)
+get_max_forced_width (GdkScreen *screen)
 {
-	return (gdk_screen_width () * 90) / 100;
+	return (gdk_screen_get_width (screen) * 90) / 100;
 }
 
 static void
 set_initial_window_geometry (NautilusWindow *window)
 {
+	GdkScreen *screen;
 	guint max_width_for_screen, max_height_for_screen;
 
+	screen = gtk_window_get_screen (GTK_WINDOW (window));
+	
 	/* Don't let GTK determine the minimum size
 	 * automatically. It will insist that the window be
 	 * really wide based on some misguided notion about
@@ -492,9 +501,9 @@ set_initial_window_geometry (NautilusWindow *window)
 	 * probably remove this broken set_size_request() here.
 	 * - hp@redhat.com
 	 */
-	
-	max_width_for_screen = get_max_forced_width ();
-	max_height_for_screen = get_max_forced_height ();
+
+	max_width_for_screen = get_max_forced_width (screen);
+	max_height_for_screen = get_max_forced_height (screen);
 
 	gtk_widget_set_size_request (GTK_WIDGET (window), 
 				     MIN (NAUTILUS_WINDOW_MIN_WIDTH, 
@@ -529,29 +538,146 @@ set_dummy_initial_view_as_menu (NautilusWindow *window)
                                   new_menu);
 }
 
+static void
+side_pane_close_requested_callback (GtkWidget *widget,
+				    gpointer user_data)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (user_data);
+
+	nautilus_window_hide_sidebar (window);
+}
+
+static void
+side_pane_size_allocate_callback (GtkWidget *widget,
+				  GtkAllocation *allocation,
+				  gpointer user_data)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (user_data);
+	
+	if (allocation->width != window->details->side_pane_width) {
+		window->details->side_pane_width = allocation->width;
+		eel_preferences_set_integer
+			(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH, 
+			 allocation->width);
+	}
+}
+
+static void
+setup_side_pane_width (NautilusWindow *window)
+{
+	static gboolean setup_auto_value= TRUE;
+
+	g_return_if_fail (window->sidebar != NULL);
+	
+	if (setup_auto_value) {
+		setup_auto_value = FALSE;
+		eel_preferences_add_auto_integer 
+			(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH,
+			 &side_pane_width_auto_value);
+	}
+
+	window->details->side_pane_width = side_pane_width_auto_value;
+
+	/* FIXME bugzilla.gnome.org 41245: Saved in pixels instead of in %? */
+        /* FIXME bugzilla.gnome.org 41245: No reality check on the value? */
+	
+	gtk_paned_set_position (GTK_PANED (window->content_hbox), 
+				side_pane_width_auto_value);
+}
+
+static void
+side_panel_set_open (GtkWidget *view,
+		     gboolean open)
+{
+	CORBA_Environment ev;
+	Bonobo_PropertyBag property_bag;
+	Bonobo_Control control;
+	
+	if (!view || !NAUTILUS_IS_VIEW_FRAME (view)) {
+		return;
+	}
+
+	control = nautilus_view_frame_get_control (NAUTILUS_VIEW_FRAME (view));
+	
+	if (control != CORBA_OBJECT_NIL) {
+		CORBA_exception_init (&ev);
+		property_bag = Bonobo_Control_getProperties (control, &ev);
+		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {
+			/* For some reason this was implemented as 'close'
+			 * before, but open seems more natural */
+			bonobo_property_bag_client_set_value_gboolean
+				(property_bag, "close", !open, &ev);
+			bonobo_object_release_unref (property_bag, NULL);
+		}
+	}
+}
+
+static void
+side_pane_switch_page_callback (NautilusSidePane *side_pane,
+				GtkWidget *panel,
+				NautilusWindow *window)
+{
+	if (window->details->current_side_panel) {
+		side_panel_set_open (window->details->current_side_panel, 
+				     FALSE);
+		eel_remove_weak_pointer (&window->details->current_side_panel);
+	}
+
+	side_panel_set_open (panel, TRUE);	
+	window->details->current_side_panel = panel;
+	eel_add_weak_pointer (&window->details->current_side_panel);
+}
+
 static void
 nautilus_window_set_up_sidebar (NautilusWindow *window)
 {
-	window->sidebar = nautilus_sidebar_new ();
+	window->sidebar = nautilus_side_pane_new ();
+
+	g_signal_connect (window->sidebar,
+			  "close_requested",
+			  G_CALLBACK (side_pane_close_requested_callback),
+			  window);
+
+	g_signal_connect (window->sidebar,
+			  "switch_page",
+			  G_CALLBACK (side_pane_switch_page_callback),
+			  window);
 
-	if (window->details->location != NULL &&
-	    window->details->title != NULL) {
-		nautilus_sidebar_set_uri (window->sidebar,
-					  window->details->location,
-					  window->details->title);
-	}
-	
-	g_signal_connect_object (window->sidebar, "location_changed",
-				 G_CALLBACK (go_to_callback), window, 0);
 	gtk_paned_pack1 (GTK_PANED (window->content_hbox),
 			 GTK_WIDGET (window->sidebar),
 			 FALSE, TRUE);
+
+	setup_side_pane_width (window);
+	g_signal_connect (window->sidebar, 
+			  "size_allocate",
+			  G_CALLBACK (side_pane_size_allocate_callback),
+			  window);
 	
-	nautilus_sidebar_setup_width (window->sidebar);
+	window->information_panel = nautilus_information_panel_new ();
+	
+	if (window->details->location != NULL &&
+	    window->details->title != NULL) {
+		nautilus_information_panel_set_uri (window->information_panel,
+						    window->details->location,
+						    window->details->title);
+	}
+
+	g_signal_connect_object (window->information_panel, "location_changed",
+				 G_CALLBACK (go_to_callback), window, 0);
+
+	gtk_widget_show (GTK_WIDGET (window->information_panel));
+
+	nautilus_side_pane_add_panel (NAUTILUS_SIDE_PANE (window->sidebar), 
+				      GTK_WIDGET (window->information_panel),
+				      _("Information"));
 
 	/* Set up the sidebar panels. */
 	update_sidebar_panels_from_preferences (window);
-
+	
 	gtk_widget_show (GTK_WIDGET (window->sidebar));
 }
 
@@ -561,6 +687,7 @@ nautilus_window_tear_down_sidebar (NautilusWindow *window)
 	nautilus_window_set_sidebar_panels (window, NULL);
 	gtk_widget_destroy (GTK_WIDGET (window->sidebar));
 	window->sidebar = NULL;
+	window->information_panel = NULL;
 }
 
 static void
@@ -901,6 +1028,7 @@ static void
 nautilus_window_size_request (GtkWidget		*widget,
 			      GtkRequisition	*requisition)
 {
+	GdkScreen *screen;
 	guint max_width;
 	guint max_height;
 
@@ -909,6 +1037,8 @@ nautilus_window_size_request (GtkWidget		*widget,
 
 	GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
+	screen = gtk_window_get_screen (GTK_WINDOW (widget));
+
 	/* Limit the requisition to be within 90% of the available screen 
 	 * real state.
 	 *
@@ -922,8 +1052,8 @@ nautilus_window_size_request (GtkWidget		*widget,
 	 * to be requested, should still be fixed.  This code is here only to 
 	 * prevent the extremely frustrating consequence of such bugs.
 	 */
-	max_width = get_max_forced_width ();
-	max_height = get_max_forced_height ();
+	max_width = get_max_forced_width (screen);
+	max_height = get_max_forced_height (screen);
 
 	if (requisition->width > (int) max_width) {
 		requisition->width = max_width;
@@ -1401,12 +1531,20 @@ void
 nautilus_window_add_sidebar_panel (NautilusWindow *window,
 				   NautilusViewFrame *sidebar_panel)
 {
+	char *label;
+
 	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (sidebar_panel));
-	g_return_if_fail (NAUTILUS_IS_SIDEBAR (window->sidebar));
+	g_return_if_fail (NAUTILUS_IS_SIDE_PANE (window->sidebar));
 	g_return_if_fail (g_list_find (window->sidebar_panels, sidebar_panel) == NULL);
 	
-	nautilus_sidebar_add_panel (window->sidebar, sidebar_panel);
+	label = nautilus_view_frame_get_label (sidebar_panel);
+	
+	nautilus_side_pane_add_panel (window->sidebar, 
+				      GTK_WIDGET (sidebar_panel), 
+				      label);
+
+	g_free (label);
 
 	g_object_ref (sidebar_panel);
 	window->sidebar_panels = g_list_prepend (window->sidebar_panels, sidebar_panel);
@@ -1422,7 +1560,8 @@ nautilus_window_remove_sidebar_panel (NautilusWindow *window, NautilusViewFrame
 		return;
 	}
 	
-	nautilus_sidebar_remove_panel (window->sidebar, sidebar_panel);
+	nautilus_side_pane_remove_panel (window->sidebar, 
+					 GTK_WIDGET (sidebar_panel));
 	window->sidebar_panels = g_list_remove (window->sidebar_panels, sidebar_panel);
 	g_object_unref (sidebar_panel);
 }
